function FigBeamforming2D(exportFlag)
% -------------------------------------------------------------------
% This script produces a figures similar to those in Malioutov et al.
% -------------------------------------------------------------------

% EvdB: Feb 14, 2011; Changed FigBeamforming2D to FigDOA1 in figures to
%                     match existing filenames. Also changed line styles to
%                     reflex existing plots in PDF.

% Check arguments
if nargin < 1, exportFlag = 0; end

% Get filename prefix
prefix      = mfilepath(mfilename('fullpath'));
prefixFig   = [prefix,'..',filesep,'..',filesep,'fig',filesep];
prefixCache = [prefix,'..',filesep,'cache',filesep];


% Initialize random number generator
randn('state',1); rand('state',0);

% Generate phase shifts
sensors = 20;
sources = [60,65,80,100.5,160]; % 68
W       = ULA(sensors,sources);

% Generate observation
SNR     = 10;
samples = 20;
[Y,N]   = GenerateRadioSignal(W,randn(length(sources),samples),SNR);

% Generate phase shifts for DOA grid
angles = 1:1:180;
Wgrid = ULA(sensors,angles);

% Compute spectrum using the Capon and MUSIC methods
Pbf  = SpectrumBF(Y,Wgrid);
Pcap = SpectrumCapon(Y,Wgrid);
Pmus = SpectrumMusic(Y,Wgrid,length(sources));

% Solve using MMV
filename = [prefixCache,'FigBeamforming2D_MMV.mat'];
if exist(filename)
   data = load(filename);
   Pmmv = data.Pmmv;
else
   [Pmmv,info] = SpectrumMMV(Y,Wgrid,11,'cvx_sedumi');
   save(filename,'Pmmv');
end

% Solve using BPDN
filename = [prefixCache,'FigBeamforming2D_L1.mat'];
if exist(filename)
   data = load(filename);
   Pl1  = data.Pl1;
else
   [Pl1,info] = SpectrumL1(Y,Wgrid,0.62,'cvx_sedumi');
   save(filename,'Pl1');
end

% Plot settings
fs = 16; % Font size

% Plot power in dB
figure(1); clf; gca; hold on;
plot(angles,Pbf,'k-',...
     angles,Pcap,'r-',...
     angles,Pmus,'g-',...
     angles,Pl1,'b-',...
     angles,Pmmv,'m-');
set(gca,'Fontsize',fs); box on;
legend('Beamforming','Capon','MUSIC','BPDN','MMV','Location','Southwest');
xlim([0,180]); xlabel('DOA (degrees)'); ylabel('Power (dB)');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigDOA1a.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigDOA1a.eps');
end


% Zoom #1
figure(1); clf; gca; hold on;
plot(angles,Pbf,'k-',...
     angles,Pcap,'r-',...
     angles,Pmus,'g-',...
     angles,Pl1,'b-',...
     angles,Pmmv,'m-');
xlim([50,110]); ylim([-22,8]); set(gca,'Fontsize',fs);  box on;
legend('Beamforming','Capon','MUSIC','BPDN','MMV','Location','Northeast');
xlabel('DOA (degrees)'); ylabel('Power (dB)');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigDOA1b.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigDOA1b.eps');
end

% Zoom #2
figure(1); clf; gca; hold on;
plot(angles,Pbf,'k-',...
     angles,Pcap,'r-',...
     angles,Pmus,'g-',...
     angles,Pl1,'b-',...
     angles,Pmmv,'m-');
xlim([145,175]); ylim([-22,8]); set(gca,'Fontsize',fs);  box on;
legend('Beamforming','Capon','MUSIC','BPDN','MMV','Location','Northeast');
xlabel('DOA (degrees)'); ylabel('Power (dB)');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigDOA1c.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigDOA1c.eps');
end


figure(1); clf; gca; hold on; fs = 12; set(gca,'Fontsize',fs);

for i=-80:20:0
  h = plot(cos(pi*angles/180)*(i+85),sin(pi*angles/180)*(i+85),'b-');
  set(h,'Color',[0.8,0.8,0.8]);
end

for i=10:10:170
  h = plot([0,cos(pi*i/180)*86],[0,sin(pi*i/180)*86],'b-');
  set(h,'Color',[0.8,0.8,0.8]);
  
  if ~ismember(i,[60,80,100,160])
     h = text(cos(pi*i/180)*90,sin(pi*i/180)*90,sprintf('%d',i));
     set(h,'HorizontalAlignment','center','VerticalAlignment','middle', ...
           'Fontsize',fs,'Color',[0.8,0.8,0.8]);
  end
  
end

for i=1:length(sources)
  h = plot([0,cos(pi*sources(i)/180)*100],...
           [0,sin(pi*sources(i)/180)*100],'k-');
  h = plot(cos(pi*sources(i)/180)*100,...
           sin(pi*sources(i)/180)*100,'k*');
end
  
% styles = {'k--','r-.','.g:','b-','m-'};
styles = {'k-','r-','g-','b-','m-'};

hLines = ...
plot(cos(pi*angles/180).*(Pbf'+85),...
     sin(pi*angles/180).*(Pbf'+85),styles{1}, ...
     cos(pi*angles/180).*(Pcap'+85),...
     sin(pi*angles/180).*(Pcap'+85),styles{2}, ...
     cos(pi*angles/180).*(Pmus'+85),...
     sin(pi*angles/180).*(Pmus'+85),styles{3}, ...
     cos(pi*angles/180).*(Pl1'+85),...
     sin(pi*angles/180).*(Pl1'+85),styles{4}, ...
     cos(pi*angles/180).*(Pmmv'+85),...
     sin(pi*angles/180).*(Pmmv'+85),styles{5});

xlim([-100,100]);
xt= [-85:20:-5,5:20:85];
x = repmat(' ', length(xt),3);

set(gca,'XTick',[-85:20:-5,5:20:85]);
for i=1:10
   x(i,:) = sprintf('%-3d',abs(xt(i))-85);
end
set(gca,'XTickLabel',x);

ylim([0,110]); set(gca,'YTick',[]);
xlabel('Power (dB)');

legend(hLines,'Beamforming','Capon','MUSIC','BPDN','MMV','Location','Northwest');
box on;

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigDOA1d.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigDOA1d.eps');
end

