function FigIterativelyRefine(figIdx,exportFlag)
% Three dimensional test

% Check export flag
if nargin < 1, figIdx = []; end;
if nargin < 2, exportFlag = false; end;

% Get filename prefix
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
prefixFig   = [prefix,'..',filesep,'..',filesep,'fig',filesep];

% Set colormap
clrmap   = [1,1,1;prism(128)];
clrmap   = [1,1,1; 1 - power(linspace(1,0,128),4)',zeros(128,1),1 - power(linspace(0,1,128),4)'];

softblue = [0.6,0.7,1] * 0.8;

% Set parameters
nSensors           = 80;
nSources           = 8;
nMeasurements      = 2;
nRefinements       = 5;   % Number of grid refinements
nDirectionsFine    = 600; % Actual directions
nDirectionsCoarse  = 100; % Meshed directions
wavelengthRelative = 5;   % Wavelength as a multiple of minimum sensor distance

if isempty(figIdx), figIdx = 1:(2*nRefinements+2); end;

% Load sensor array
[S,distMin] = CirclePacking(nSensors);
S(3,:) = 0;

% Compute wavelength based on minimal sensor distance
wavelength = wavelengthRelative * distMin;

% Load high-resolution direction-of-arrival grid
Dfine = PointArrangement(3,2*nDirectionsFine,1e-5);
idx = Dfine(3,:) >= 0; Dfine = Dfine(:,idx);
nDfine = size(Dfine,2);

% Load low-resolution direction-of-arrival grid
Dcoarse = PointArrangement(3,2*nDirectionsCoarse,1e-5);
idx = Dcoarse(3,:) >= 0; Dcoarse = Dcoarse(:,idx);
nDcoarse = size(Dcoarse,2);

% Add circle at base
nAngle = ceil(sqrt(nDcoarse));
Angle = 2*pi*(0:nAngle-1)/nAngle;
Dcoarse = [Dcoarse,[cos(Angle);sin(Angle);zeros(1,nAngle)]];
nDcoarse = nDcoarse + nAngle;

% Compute actual complex shift factors
Afine   = ComputeShifts3D(S,Dfine,wavelength);

% Generate sparse complex signal coefficients
rand('state',nDfine + nSensors); randn('state',nDfine + nSensors);
p  = randperm(nDfine);
X0 = zeros(nDfine,nMeasurements);
X0(p(1:nSources),:) = randn(nSources,nMeasurements) + sqrt(-1) * randn(nSources,nMeasurements);

% Generate signal
B = Afine*X0;

% Plot actual signal
if ismember(1,figIdx)
   figure(1); clf; colormap(clrmap);
   h = PlotSphereSignal(Dfine,X0(:,1));
   set(h,'EdgeColor',softblue);
   %for i=1:length(h)
   %   set(h(i),'EdgeColor',get(h(i),'FaceColor'))
   %end

   axis off; axis equal; drawnow;
   if exportFlag == 1
      ExportFigurePNG(prefixFig,'FigIterativelyRefine_Original.png');
   elseif exportFlag == 2
      ExportFigureTIFF2EPS(prefixFig,'FigIterativelyRefine_Original.eps');
   end

   if length(figIdx) == 1
      return;
   end
end

% Recover using actual grid
filename = [prefixCache,'FigIterativelyRefine_0.mat'];
if exist(filename)
   data = load(filename);
   XMMV = data.XMMV;
else
   [XMMV,info] = CVXL12(Afine,B,0,'SDPT3');
   save(filename,'XMMV');
end

% Plot recovery
if ismember(2,figIdx)
   figure(2); clf; colormap(clrmap);
   h = PlotSphereSignal(Dfine,XMMV(:,1));
   set(h,'EdgeColor',[0.7,0.75,1]);
   axis off; axis equal; drawnow;
   if exportFlag == 1
      ExportFigurePNG(prefixFig,'FigIterativelyRefine_ExactGrid.png');
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,'FigIterativelyRefine_ExactGrid.eps');
   end
end

for iter=1:nRefinements
   % Compute complex shift factors on grid
   Acoarse = ComputeShifts3D(S,Dcoarse,wavelength);

   % Plot current grid
   if ismember(2*iter+1,figIdx)
      figure(3); clf; gca; colormap(clrmap); hold on;
      h = plot3(Dcoarse(1,:),Dcoarse(2,:),2*Dcoarse(3,:),'b*');
      h = PlotSphereSignal(Dcoarse,zeros(size(Dcoarse,2),1));
      set(h,'FaceColor',[1,1,1]);
      hold off; axis off; axis equal; drawnow;
      if exportFlag == 1
         ExportFigurePNG(prefixFig,sprintf('FigIterativelyRefine_NewGrid_%d.png',iter));
      elseif exportFlag == 2
         ExportFigureEPS(prefixFig,sprintf('FigIterativelyRefine_NewGrid_%d.eps',iter));
      end
   end

   % Solve MMV problem
   filename = [prefixCache,sprintf('FigIterativelyRefine_%d.mat',iter)];
   if exist(filename)
      data = load(filename);
      XMMV = data.XMMV;
   else
      [XMMV,info] = CVXL12(Acoarse,B,5/(iter^2),'SDPT3');
      save(filename,'XMMV');
   end

   % Plot recovered signal
   if ismember(2*iter+2,figIdx)
      figure(4); clf; colormap(clrmap);
      h = PlotSphereSignal(Dcoarse,XMMV(:,1));
      set(h,'EdgeColor',softblue);
      axis off; axis equal; drawnow;
      if exportFlag == 1
         ExportFigurePNG(prefixFig,sprintf('FigIterativelyRefine_Recovered_%d.png',iter));
      elseif exportFlag == 2
         ExportFigureTIFF2EPS(prefixFig,sprintf('FigIterativelyRefine_Recovered_%d.eps',iter));
      end
   end

   % Refine grid: For each face check if one of the nodes is active
   active = abs(XMMV(:,1)) > 0.05;
   H = convhulln(Dcoarse');
   for i=1:size(H,1)
      if any(active(H(i,:)))
         v = sum(Dcoarse(:,H(i,:)),2);
         v = v / norm(v,2);
         Dcoarse(:,end+1) = v;
      end
   end
   nDcoarse = size(Dcoarse,2);


   if ~exportFlag && (iter ~= nRefinements)
      fprintf('Press <return> to continue . . .\n');
      pause;
   end
end
