function FigMMVGaussian2(s,plotIdx,exportFlag)

% Check export flag
if nargin < 2, plotIdx = [1,2,3,4]; end;
if nargin < 3, exportFlag = false; end;

% Get filename prefix
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
prefixFig   = [prefix,'..',filesep,'..',filesep,'fig',filesep];

% Set plotting parameters
fontsize = 32;

% Set problem size
m = 50; n = 200; 

if s == 20
  recoverIdx = [4]; failIdx = [8];
else
  recoverIdx = []; failIdx = [];
end


kMax    = 20;
nTrials = 10;
sigmaFraction = power(linspace(0,1,16),3);
sigmaFraction = sigmaFraction(15:-1:1);


% Construct filename
filename = sprintf('%sFigMMVGaussian2_%d_%d_%d.mat',prefixCache,m,n,s);

% Load file if it exists
if exist(filename)
   f = load(filename);
   runtimeCVX = f.runtimeCVX;
   runtimeSPG = f.runtimeSPG;
   runtimePQN = f.runtimePQN;
   funevalSPG = f.funevalSPG;
   funevalPQN = f.funevalPQN;

   % Enlarge array if needed
   if size(runtimeCVX,2) < kMax
      runtimeCVX(:,size(runtimeCVX,2)+1:kMax,:) = NaN;
      runtimeSPG(:,size(runtimeCVX,2)+1:kMax,:) = NaN;
      runtimePQN(:,size(runtimeCVX,2)+1:kMax,:) = NaN;
      funevalSPG(:,size(runtimeCVX,2)+1:kMax,:) = NaN;
      funevalPQN(:,size(runtimeCVX,2)+1:kMax,:) = NaN;
   end
   if size(runtimeCVX,3) < nTrials
      runtimeCVX(:,:,size(runtimeCVX,3)+1:nTrial) = NaN;
      runtimeSPG(:,:,size(runtimeCVX,3)+1:nTrial) = NaN;
      runtimePQN(:,:,size(runtimeCVX,3)+1:nTrial) = NaN;
      funevalSPG(:,:,size(runtimeCVX,3)+1:nTrial) = NaN;
      funevalPQN(:,:,size(runtimeCVX,3)+1:nTrial) = NaN;
   end
else
   initValues = NaN * ones(length(sigmaFraction),kMax,nTrials);
   runtimeCVX = initValues;
   runtimeSPG = initValues;
   runtimePQN = initValues;
   funevalSPG = initValues;
   funevalPQN = initValues;
end

for i=1:nTrials
   randn('state',i);
   A = randn(m,n);
   X0 = zeros(n,kMax); X0(1:s,:) = randn(s,kMax);
   B0 = A*X0;
   
   for j=1:length(sigmaFraction)
      
      for k=1:kMax
         sigma = sigmaFraction(j) * norm(B0(:,1:k),'fro');
         change = 0;
         Z = randn(size(B0));
         Z = sigma * Z / norm(Z,'fro');
         B = B0 + Z;

         if isnan(runtimeCVX(j,k,i))
            cvx_precision low
            [X,info] = cvx_mmv(A,B(:,1:k),sigma);
            runtimeCVX(j,k,i) = info.time_solve;
            change = 1;
         end

         if isnan(runtimeSPG(j,k,i))
            options.solver = 1;
            [X,info] = gbpdn_mmv(A,B(:,1:k),sigma,options);
            runtimeSPG(j,k,i) = info.timeTotal;
            funevalSPG(j,k,i) = info.nProdA + info.nProdAt;
            change = 1;
         end

         if isnan(runtimePQN(j,k,i))
            options.solver = 2;
            [X,info] = gbpdn_mmv(A,B(:,1:k),sigma,options);
            runtimePQN(j,k,i) = info.timeTotal;
            funevalPQN(j,k,i) = info.nProdA + info.nProdAt;
            change = 1;
         end

         % Save result
         if change
            save(filename,'runtimeCVX','runtimeSPG','runtimePQN','funevalSPG','funevalPQN');
         end
      end
   end
end


% ---------------------------------------------------------------------
% Plot results
% ---------------------------------------------------------------------
if ismember(1,plotIdx)
   plotIdx = setdiff(plotIdx,1);

   figure(1); clf; gca;
   surf(1:kMax,sigmaFraction,mean(runtimeCVX,3)); colormap(jet);
   set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
   xlabel('k'); ylabel('\sigma'); zlabel('time (sec.)');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%da.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%da.eps',m,n,s));
   end

   figure(2); clf; gca;
   surf(1:kMax,sigmaFraction,mean(runtimeSPG,3)); colormap(jet);
   set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
   xlabel('k'); ylabel('\sigma'); zlabel('time (sec.)');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%db.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%db.eps',m,n,s));
   end

   figure(3); clf; gca;
   surf(1:kMax,sigmaFraction,mean(runtimePQN,3)); colormap(jet);
   set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
   xlabel('k'); ylabel('\sigma'); zlabel('time (sec.)');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dc.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dc.eps',m,n,s));
   end

   if ~exportFlag && ~isempty(plotIdx)
      disp('Press <return> to continue . . .'); pause;
   end
end

if ismember(2,plotIdx)
   plotIdx = setdiff(plotIdx,2);

   figure(1); clf;
   h = plot(mean(mean(runtimeCVX,3),1)); set(h,'LineWidth',2);
   set(gca,'Fontsize',fontsize,'YScale','log'); xlabel('k'); ylabel('time (sec.)');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dd.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dd.eps',m,n,s));
   end

   figure(2); clf;
   h = plot(mean(mean(runtimeSPG,3),2)); set(h,'LineWidth',2);
   set(gca,'Fontsize',fontsize,'YScale','log'); xlabel('i'); ylabel('time (sec.)');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%de.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%de.eps',m,n,s));
   end

   % Close figure 3 if needed
   if ishandle(3), close(3); end;

   if ~exportFlag && ~isempty(plotIdx)
      disp('Press <return> to continue . . .'); pause;
   end
end

if ismember(3,plotIdx)
   plotIdx = setdiff(plotIdx,3);

   % Function evaluations
   figure(1); clf;
   surf(1:kMax,sigmaFraction,mean(funevalSPG,3)); colormap(jet);
   set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
   xlabel('k'); ylabel('\sigma'); zlabel('function eval.');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%df.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%df.eps',m,n,s));
   end

   figure(2); clf;
   surf(1:kMax,sigmaFraction,mean(funevalPQN,3)); colormap(jet);
   set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
   xlabel('k'); ylabel('\sigma'); zlabel('function eval.');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dg.pdf',m,n,s));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dg.eps',m,n,s));
   end

   % Close figure 3 if needed
   if ishandle(3), close(3); end;

   if ~exportFlag && ~isempty(plotIdx)
      disp('Press <return> to continue . . .'); pause;
   end
end


if ismember(4,plotIdx)
   plotIdx = setdiff(plotIdx,4);

   % Close figure 2 and 3 if needed
   if ishandle(2), close(2); end;
   if ishandle(3), close(3); end;

   % Runtime when recovery succeeds
   for idx=recoverIdx
      figure(1); clf;
      surf(1:kMax,sigmaFraction,runtimeSPG(:,:,idx)); colormap(jet);
      set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
      xlabel('k'); ylabel('\sigma'); zlabel('time (sec.)');

      if exportFlag == 1
         ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dh%d.pdf',m,n,s,idx));
      elseif exportFlag == 2
         ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%dh%d.eps',m,n,s,idx));
      else
         disp('Press <return> to continue . . .'); pause;
      end
   end

   % Runtime when recovery fails
   for idx=failIdx
      figure(1); clf;
      surf(1:kMax,sigmaFraction,runtimeSPG(:,:,idx)); colormap(jet);
      set(gca,'Fontsize',fontsize,'yscale','log','ydir','reverse');
      xlabel('k'); ylabel('\sigma'); zlabel('time (sec.)');

      if exportFlag == 1
         ExportFigurePDF(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%di%d.pdf',m,n,s,idx));
      elseif exportFlag == 2
         ExportFigureEPS(prefixFig,sprintf('FigMMVGaussian2_%d_%d_%di%d.eps',m,n,s,idx));
      else
         disp('Press <return> to continue . . .'); pause;
      end
   end
end
