function X = PointArrangement(d,n,target_precision);
% POINTARRANGEMENT  Approximate minimal energy arrangement of points on sphere
%
%    X = POINTARRANGEMENT(D,N,PRECISION) generates a set of N random points on
%    the (D-1)-Sphere and approximately minimizes the potential energy by means
%    of mutual repulsion. The process stops when the maximum distance moved by
%    any of the points is below PRECISION.

% References:
% [1] R. H. Hardin, N. J. A. Sloane and W. D. Smith
%     Minimal Energy Arrangements of Points on a Sphere
%     http://www.research.att.com/~njas/electrons/
% [2] R. H. Hardin, N. J. A. Sloane and W. D. Smith
%     Spherical Codes, book in preparation.

% Construct filename
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
filename    = sprintf('%sPointArrangement_%d_%d.mat',prefixCache,d,n);

% Load file if it exists
if exist(filename)
   s = load(filename);
   X = s.X;
   precision = s.precision;
   clear s;
else
   state = randn('state'); randn('state',0);
   X = randn(d,n); X = X * spdiags(1./sqrt(sum(X.^2,1)'),0,n,n);
   X(:,1) = [zeros(d-1,1);1];
   randn('state',state);

   precision = power(2,d);
end

% Start iterating
while precision > target_precision
   force = zeros(d,n);
   for i=1:n
      for j=i+1:n
         diff = X(:,i) - X(:,j);
         dist = norm(diff,2);
         force(:,i) = force(:,i) + diff/dist;
         force(:,j) = force(:,j) - diff/dist;
      end
   end
   force(:,1) = zeros(1,d);

   % Update positions and compute maximum displacement
   Xprev = X;
   X = X + force;
   X = X * spdiags(1./sqrt(sum(X.^2,1)'),0,n,n);
   precision = max(sqrt(sum((Xprev - X).^2)));

   % Store result
   save(filename,'X','precision');

   % Display progress
   disp(precision);
end

