function [X,info] = SeDuMiL12(A,B,sigma);
% Solve using SeDuMi:
%
% Minimize ||X||_1,2 subject to ||AX-B||_F <= sigma
%

[m,n] = size(A);
k     = size(B,2);
info  = struct();

% Formulate as:
%
% Minimize c^Ts   subject to   Ms = b, cone(s)
%

% s = [r0             # 1
%      s0             # 1
%      real(r(1:m*k)) # m*k
%      imag(r(1:m*k)) # m*k
%      - - - - - 
%      s1             # 1  \
%      x(1,1:k)       # 2k  |
%      - - - - -            |
%      s2             # 1   |
%      x(2,1:k)       # 2k   \  n times
%      - - - - -             /
%      :                    |
%      - - - - -            |
%      sn             # 1   |
%      x(n,1:k)];     # 2k /
%
% Real and imaginary parts of x(n,1:k) are split as
% real(n,1), imag(n,1), real(n,2) ...
%
% Note that matrix X has to be vectorized row by row, rather than
% column by column;
%
% Minimize s1+s2+...+sn   subject to   A*X + R = B
%                                      norm(R,'fro') <= s0
%                                      norm(x(i,:),2) <= si
%                                      s0+r0   = sigma
%                                      r0 >= 0;

% Start preparation time
tic;

% Pre-allocate matrices and vectors
M = zeros(2*m*k+1,2+2*m*k+n*(2*k+1)); % Using sparse is too slow
b = zeros(2*m*k+1,1);

% Add constraint s0 = sigma
b(1)   = sigma;
M(1,1:2) = 1; % s0 + r0 = sigma

% Add constraints A * X(:,i) + R(:,i) = B(:,i)
b(2:end) = [real(B(:));imag(B(:))];
for i=1:k
   M(1+(i-1)*m+(1:m),     2+(0:m-1)*k+i)               = speye(m,m);  % I*R
   M(1+(i-1)*m+(1:m),     2+2*k*m+(0:n-1)*(2*k+1)+2*i) = real(A);     % A*X
   M(1+(i-1)*m+(1:m),     3+2*k*m+(0:n-1)*(2*k+1)+2*i) =-imag(A);
   M(1+(i-1)*m+(1:m)+k*m, 2+(0:m-1)*k+m*k+i)           = speye(m,m);  % I*R
   M(1+(i-1)*m+(1:m)+k*m, 2+2*k*m+(0:n-1)*(2*k+1)+2*i) = imag(A);     % A*X
   M(1+(i-1)*m+(1:m)+k*m, 3+2*k*m+(0:n-1)*(2*k+1)+2*i) = real(A);
end

% Construct weight vector c
c = zeros(2+2*m*k+n*(2*k+1),1);
c(3 + 2*m*k + (2*k+1) * (0:n-1)) = 1;

% Construct cone information structure
cone = struct();
cone.f = 0;                            % Number of free variables
cone.l = 1;                            % Number of nonnegative variables
cone.q = [2*m*k+1, ones(1,n)*(2*k+1)]; % Number of quadratic terms
cone.r = 0;                            % Number of rotated quadratic
cone.s = 0;                            % Number of semi-definite variables

% End preparation time
info.time_prepare = toc;

% Call SeDuMi
tic
[x,y,sedumi_info] = sedumi(M,b,c,cone);
info.time_solve = toc;

% Extract solution
X = zeros(n,k);
for i=1:n
   X(i,:) = x(3+2*m*k+(i-1)*(2*k+1)+(1:2:2*k))' + ...
            x(3+2*m*k+(i-1)*(2*k+1)+(2:2:2*k))' * sqrt(-1);
end
