function [p,info] = SpectrumMMV(Y,W,sigma,solver);
% Power spectrum using MMV, solve
%
%    Minimize  ||X||_1,2  subject to  ||WX - Y||_F <= sigma
%
% Y are the sensor measurements, W are the phase shifts for a
% discrete grid of directions of arrival.
info = struct();

% Solve MMV problem
switch lower(solver)
  case {'spg'}
      options = struct();
      options.decTol     = 1e-9;
      options.bpTol      = 1e-7;
      options.optTol     = 1e-7;
      options.iterations = 1e5;
      
      [X,r,g,spg_info]   = spg_mmv(W,Y,sigma,options);
      
      info.time_solve    = spg_info.timeTotal;
      info.time_prepare  = 0;
      
 case {'pqn'}
      options = struct();
      options.optTol     = 1e-4;
      options.verbosity  = 3;

      [X,r,g,pqn_info]   = pqn_mmv(W,Y,sigma,options);
      
      info.time_solve    = pqn_info.timeTotal;
      info.time_prepare  = 0;
  
      
  case {'sedumi'}
      [X,info] = SeDuMiL12(W,Y,sigma);
  
  case {'cvx'}
      [X,info] = CVXL12(W,Y,sigma,'SDPT3');
      
  case {'cvx_sedumi'}
      [X,info] = CVXL12(W,Y,sigma,'SeDuMi');
 
  otherwise
      error('Unknown solver %s',solver);
end

% Compute feasible dual point
R = Y - W * X;
info.rNorm = norm(R,'fro');
info.xNorm = sum(sqrt(sum(abs(X).^2,2)));
info.time  = info.time_solve + info.time_prepare;

% Compute power in dB
Pmmv = sqrt(sum(X.^2,2));
Pmmv = Pmmv / max(abs(Pmmv));
p    = 10*log10(abs(Pmmv(:))+1e-8);

