function FigSpectMixture(exportFlag)

% Check arguments
if nargin < 1, exportFlag = 0; end;

fontsize = 24;

% Get path information
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
prefixFig   = [prefix,'..',filesep,'..',filesep,'fig',filesep];

% ---------------------------------------------------------------------------
% Construct mixture
% ---------------------------------------------------------------------------
info = GenerateSpectMixture;
dictionary = info.dictionary;
idx        = info.idx;
mz         = info.mz;
A          = info.A;
b          = info.b;
x0         = info.x0;
noise      = info.noise;
n          = length(idx);

% Load precomputed result (if available)
filename = [prefixCache, 'FigMixtureCVX.mat'];
if exist(filename)
   data = load(filename);
   x1   = data.x1;
else
   [x1,info] = nonneg_cvx(A,b,norm(noise,2) + 1e-6*norm(b));

   save(filename,'x1');
end

%options.solver = 1; % Use SPG
%[x2,info] = gbpdn_srbpdn(A,b,ones(size(A,2),1),1e-6*norm(b),options);

filename = [prefixCache, 'FigMixtureSPG.mat'];
if exist(filename)
   data = load(filename);
   x2   = data.x2;
else
   options.verbosity = 3;
   [x2,info] = nonneg_spg(A,b,norm(noise,2) + 1e-6*norm(b),options);

   save(filename,'x2');
end

% Print list of components
fprintf('\n');
fprintf('----------------------------------------------------------\n');
fprintf('Composition of mixture (%d/%d)\n',nnz(x0),size(A,2));
fprintf('%6s  %6s  %6s  %s\n','Actual','SDPT3','SPG','Name');
fprintf('----------------------------------------------------------\n');
p = find(x0);
for i = 1:length(p)
   fprintf('%5.2f%%  %6.2f%%  %6.2f%%  %s\n', 100*x0(p(i)), 100*x1(p(i)), 100*x2(p(i)), dictionary{idx(p(i)),2});
end
fprintf('----------------------------------------------------------\n');


% Plot mass spectum of mixture
figure(1);
stemMassSpectrum(mz,b,fontsize);

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigSpectMixture.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigSpectMixture.eps');
end


% Plot mass spectra of first six components
figure(2);
p = find(x0);
for i = 1:6
   data = dictionary{idx(p(i)),4};
   stemMassSpectrum(data(1,:),data(2,:),fontsize);

   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigSpectComponent%d.pdf',i));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigSpectComponent%d.eps',i));
   elseif i ~= 6
      fprintf('Press <return> to continue . . .\n');
      pause;
   end
end

% Plot mass spectra of first six components (combined)
% figure(3); clf; gca; hold on;
% markers = [];
% for i = 1:6
%    data = dictionary{idx(p(i)),4};
%    m    = plotMassSpectrum(data(1,:),data(2,:),i-1,[0,90]);
%    markers = [markers,m];
% end
% hold off;
% set(gca,'YTick',markers,'YTickLabel',[]);


% Noisy (additive nonnegative noise only)
noise = abs(randn(size(A,1),1))*.1;
bn = A*x0 + noise;

filename = [prefixCache, 'FigMixtureCVXNoisy.mat'];
if exist(filename)
   data = load(filename);
   x2   = data.x2;
else
   [x2,info] = nonneg_cvx(A,bn,norm(noise,2) + 1e-10*norm(bn));

   save(filename,'x2');
end


% Plot original and recovered coefficients
figure(1);
j = find(x0 > 1e-4); h1 = plot(j,100*x0(j),'bo'); hold on;
j = find(x1 > 1e-4); h2 = plot(j,100*x1(j),'r+');
j = find(x2 > 1e-4); h3 = plot(j,100*x2(j),'r.'); hold off
set(gca,'Fontsize',fontsize);
set([h1,h2,h3],'Markersize',12);
xlabel('Component')
ylabel('Percentage')
legend('Actual','Recovered','Recovered (Noisy)');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigCoeffMixture.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigCoeffMixture.eps');
end

% figure(5);
% stem(mz,b);

% figure(6);
% plot(mz,b-bn);

% ----------------------------------------------------------------------------
function markers = plotMassSpectrum(mz,v,offset,mzRange)
% ----------------------------------------------------------------------------

v = 100 * v / max(v);
offset = offset * 140;
markers = offset + [0,25,50,75,100];

for i=1:length(v)
   h = plot(mzRange,[offset,offset],'r-');
   set(h,'Color',[0.7,0.7,0.9]);
   if v(i) > 0.01
      h = plot([mz(i),mz(i)],offset+[0,v(i)],'b-');
   end
end


for i=[25,50,75,100]
   h = plot(mzRange,[offset+i,offset+i],'r:');
   set(h,'Color',[0.7,0.7,0.9]);
end


% ----------------------------------------------------------------------------
function stemMassSpectrum(mz,v,fontsize)
% ----------------------------------------------------------------------------

if nargin < 3, fontsize = 20; end;

v = 100 * v / max(v);

h = stem(mz,v,'.b-');
set(h,'LineWidth',2,'Marker','none');
set(gca,'Fontsize',fontsize);
xlabel('m/z');
ylabel('Relative intensity');

