function FigPareto(exportFlag)

if nargin < 1, exportFlag = 0; end;
prefix      = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
prefixFig   = [prefix,'..',filesep,'..',filesep,'fig',filesep];

% Set plot parameters
fontsize   = 16;
fontsizeRt = 16; % Root finding
linewidth  = 2;


% =====================================================================
% Set up problem
% =====================================================================

% Initialize random number generator
randn('state',0);

% Generate matrix A
A = randn(50,128);
[U,S,V] = svd(A);
d = repmat(power(2,1:5),10,1);
S = full(spdiags(d(:),0,size(S,1),size(S,2)));
A = U * S * V;

% Generate measurement b
b = randn(50,1);

% Load Pareto curve if available
filename = [prefixCache,'FigPareto.mat'];
if exist(filename)
   f = load(filename);
   tauValues   = f.tauValues;
   sigmaValues = f.sigmaValues;
else
   % Compute Pareto curve
   cvx_begin
      variable x(128);
      minimize norm(x,1);
      subject to
         A*x == b;
   cvx_end

   % Generate discretization of tau
   tauValues = linspace(0,norm(x,1),101);
   
   % Compute sigma values
   sigmaValues = zeros(size(tauValues));
   for i=1:length(tauValues)
      tau = tauValues(i);
         cvx_begin
            variable x(128);
               minimize norm(A*x-b,2);
               subject to
                  norm(x,1) <= tau;
         cvx_end
	 sigmaValues(i) = norm(A*x-b,2);
   end
   
   % Save curve
   save(filename,'tauValues','sigmaValues');
end


% =====================================================================
% Plot Pareto curve
% =====================================================================
figure(1); clf; gca; hold on; box on;
h = plot([tauValues,14],[sigmaValues,0],'r-');
set(h,'LineWidth',2); set(gca,'FontSize',fontsize);
set(gca,'XTick',[],'YTick',[]);
if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigParetoInit1.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigParetoInit1.eps');
else
   disp('Press <return> to continue . . .'); pause;
end

% =====================================================================
% Compute intersection of curve with given sigma
% =====================================================================
sigma = 1;
filename = [prefixCache,'FigParetoStep0.mat'];
if exist(filename)
   f = load(filename);
   x = f.x;
else
   cvx_begin
      variable x(128);
      minimize norm(x,1);
      subject to
         norm(A*x-b,2) <= sigma;
   cvx_end
   save(filename,'x');
end
tau = norm(x,1);


% =====================================================================
% Plot root-finding steps
% =====================================================================
tau = 0;

figure(1); clf; gca; hold on;
h = plot(tauValues,sigmaValues,'r-'); set(h,'LineWidth',2);
set(gca,'FontSize',fontsizeRt);
set(gca,'XTick',[],'YTick',[]);
yValues = get(gca,'ylim'); set(gca,'ylim',yValues);
xValues = get(gca,'XLim'); set(gca,'XLim',xValues);
h = plot(xValues,[sigma,sigma],'b--'); set(h,'LineWidth',2);
hTau = plot([tau,tau],yValues,'b-'); set(hTau,'LineWidth',2,'Color',[0.3,0.3,1]);
hTangent = plot([-1,-1],[2,2],'b-'); set(hTangent,'LineWidth',2,'Color',[0.3,0.3,1]);
h = plot(tauValues,sigmaValues,'r-'); set(h,'LineWidth',2);
h1 = plot(tau,-1,'b.');  set(h1,'MarkerSize',8,'LineWidth',2);
h2 = plot(tau,-1,'bo');  set(h2,'MarkerSize',8,'LineWidth',2);
hPhiMarker = [h1,h2];
h1 = plot(-1,sigma,'b.');  set(h1,'MarkerSize',8,'LineWidth',2,'Color',[0,0,1]);
h2 = plot(-1,sigma,'bo');  set(h2,'MarkerSize',8,'LineWidth',2,'Color',[0,0,1]);
hSigmaMarker = [h1,h2];
hold off; box on;


if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigPareto0.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigPareto0.eps');
end

for iteration = 1:5
   % Solve BPDN problem and compute residual
   filename = [prefixCache,sprintf('FigParetoStep%d.mat',iteration)];
   if exist(filename)
      f = load(filename);
      x = f.x;
   else
      cvx_begin
         variable x(128);
         minimize norm(A*x-b,2);
         subject to
            norm(x,1) <= tau;
      cvx_end
      save(filename,'x');
   end
   r = A*x - b; rNorm = norm(r,2);
   
   % Compute derivative
   phiPrime = -norm(A'*r / rNorm,Inf);
   tauNew = tau + (sigma - rNorm) / phiPrime;

   % Plot current point on Pareto curve
   set(hPhiMarker,'XData',[tau],'YData',rNorm,'Color',[0,0,1]);
   set(hSigmaMarker,'Color',[0,0,1]);
   set(hTau,'XData',[tau,tau]);
   set(hTangent,'Visible','off');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigPareto%da.pdf',iteration));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigPareto%da.eps',iteration));
   else
      disp('Press <return> to continue . . .'); pause;
   end

   % Plot tangent and new tau      
   set(hTangent,'Xdata',[0,tau-rNorm/phiPrime],'Ydata',[rNorm-tau*phiPrime,0]);
   set(hTangent,'Visible','on');
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigPareto%db.pdf',iteration));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigPareto%db.eps',iteration));
   else
      disp('Press <return> to continue . . .'); pause;
   end


   % Plot tangent and new tau      
   set(hSigmaMarker,'XData',tauNew,'Color',[0,0,1]);
   set(hTau,'XData',[tauNew,tauNew]);
   if exportFlag == 1
      ExportFigurePDF(prefixFig,sprintf('FigPareto%dc.pdf',iteration));
   elseif exportFlag == 2
      ExportFigureEPS(prefixFig,sprintf('FigPareto%dc.eps',iteration));
   else
      disp('Press <return> to continue . . .'); pause;
   end

   % Update tau   
   tau = tauNew;
end

% Display final residual
fprintf('Final residual: %f\n',rNorm);

