function RootConvergenceNonneg(exportFlag)

% Get input parameters
if nargin < 1, exportFlag = 0; end;

% Get path information
prefix     = mfilepath(mfilename('fullpath'));
prefixCache = [prefix,'..',filesep,'cache',filesep];
prefixFig  = [prefix,'..',filesep,'..',filesep,'fig',filesep];

% Set plotting parameters
fontsize = 24;

% Generate a random problem
seed = 16;  % 4,5,12,16,24
randn('state',seed);
A = randn(40,100);
b = randn(40,1);

sigma = norm(b,2) / 20;
maxIter = 15;


% Construct filename
filename = sprintf('%sRootConvNonneg_%d.mat',prefixCache,seed);

% Load file if it exists
if exist(filename)
   f = load(filename);

   % Extract variables
   tauX  = f.tauX;
   vTau1 = f.vTau1;  vObj1 = f.vObj1;  vOpt1 = f.vOpt1;  nIter1 = f.nIter1;
   vTau2 = f.vTau2;  vObj2 = f.vObj2;  vOpt2 = f.vOpt2;  nIter2 = f.nIter2;
   vTau3 = f.vTau3;  vObj3 = f.vObj3;  vOpt3 = f.vOpt3;  nIter3 = f.nIter3;
else
   % ==================================================================
   % Run CXV to obtain good estimate for tau
   % ==================================================================
   cvx_begin
      variables x(100);
      minimize(ones(length(x),1)' * x);
      subject to
         norm(A*x - b) <= sigma;
         x >= 0;
   cvx_end

   tauX = norm(x,1);

   options = struct();
   options.verbosity = 2;
   options.optTol    = -1;
   options.dxTol     = -1;

   % ==================================================================
   % Near exact solver (relative gap < 1e-5)
   % ==================================================================
   funGamma = @(k,r,sigma) 1e-5;
   options.callback = @stoppingCriterion1;
   [vTau1,vObj1,vOpt1,nIter1] = findroot(A,b,sigma,options,maxIter,funGamma);


   % ==================================================================
   % Inexact solve #1, tolerance decreasing
   % ==================================================================
   %tau = 0; tauArray2 = []; fArray2 = []; x = zeros(size(A,2),1);
   %for i=1:nIter
   %   options.optTol = max(1e-10,exp(-i*0.7));
   %   tauArray2(i) = tau;
   %   [x,r,g,info] = spgl1(A,b,tau,[],x,options);
   %   rNorm = norm(r);
   %   fArray2(i) = rNorm - sigma;
   %
   %   tau = tau + rNorm * (rNorm - sigma) / options.dual_norm(A'*r,1);
   %end


   % ==================================================================
   % Inexact solve, tolerance based on error
   % ==================================================================
   % Stopping criterion is based primarily on the
   % [sigma,sigma+1.05*delta] interval.
   %funGamma = @(k,r,sigma) 0; % Gamma is not used for this stopping criterion
   %options.callback = @stoppingCriterion2;
   funGamma = @(k,r,sigma) 0.9;
   options.callback = @stoppingCriterion3;
   [vTau2,vObj2,vOpt2,nIter2] = findroot(A,b,sigma,options,maxIter,funGamma);

   % Stopping criterion is based both on the [sigma,sigma+1.05*delta]
   % interval and the duality gap.
   funGamma = @(k,r,sigma) max(1e-12, power(1.2,-k));
   options.callback = @stoppingCriterion3;
   [vTau3,vObj3,vOpt3,nIter3] = findroot(A,b,sigma,options,maxIter,funGamma);

   save(filename,'tauX','vTau1','vTau2','vTau3',...
                        'vObj1','vObj2','vObj3',...
                        'vOpt1','vOpt2','vOpt3',...
                        'nIter1','nIter2','nIter3');
end

% =====================================================================
% Plot results
% =====================================================================
xlim = find(~isnan(vObj1) | ~isnan(vObj2),1,'last');

if exportFlag
   legendStr = {'Exact','Inexact'};
else
   legendStr = {'Exact','Inexact 1','Inexact 2'};
end

figure(1);
h = plot(1:maxIter,abs(vTau1-tauX)/max(1,tauX),'*b-',...
         1:maxIter,abs(vTau2-tauX)/max(1,tauX),'*r--',...
         1:maxIter,abs(vTau3-tauX)/max(1,tauX),'*m--');
set(gca,'Fontsize',fontsize,'YScale','log','Xlim',[1,xlim],'Ylim',[1e-7,1]);
if exportFlag
   set(h,'LineWidth',2);
   set(h(3),'Color',[1,0,0]);
   delete(h(2));
else
   set(h(2),'Color',[1,0.6,0.2]);
end;
legend(legendStr{:});
xlabel('k'); 
if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigRootConvNonneg1.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigRootConvNonneg1.eps');
else
   ylabel('|\tau_k - \tau^*|/ max(\tau,1)');
end


figure(2);
h = plot(1:maxIter,abs(vObj1-sigma)/max(1,sigma),'*b-',...
         1:maxIter,abs(vObj2-sigma)/max(1,sigma),'*r--',...
         1:maxIter,abs(vObj3-sigma)/max(1,sigma),'*m--');
set(gca,'Fontsize',fontsize,'YScale','log','Xlim',[1,xlim]);
if exportFlag
   set(h,'LineWidth',2);
   set(h(3),'Color',[1,0,0]);
   delete(h(2));
else
   set(h(2),'Color',[1,0.6,0.2]);
end;
legend(legendStr{:});
xlabel('k'); ylabel('|\phi_k - \sigma| / max(\sigma,1)'); 

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigRootConvNonneg2.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigRootConvNonneg2.eps');
end


figure(3);
h = plot(1:maxIter,vOpt1,'*b-', ...
         1:maxIter,vOpt2,'*r--',...
         1:maxIter,vOpt3,'*m--');
%         1:maxIter,abs(vObj2-sigma),'.r-.');
set(gca,'Fontsize',fontsize,'YScale','log','Xlim',[1,xlim]);
if exportFlag
   set(h,'LineWidth',2);
   set(h(3),'Color',[1,0,0]);
   delete(h(2));
else
   set(h(2),'Color',[1,0.6,0.2]);
end;
legend(legendStr{:});
xlabel('k'); ylabel('Duality gap');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigRootConvNonneg3.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigRootConvNonneg3.eps');
end


figure(4);
h = plot(1:maxIter,vOpt1./max(1,vObj1),'*b-', ...
         1:maxIter,vOpt2./max(1,vObj2),'*r--',...
         1:maxIter,vOpt3./max(1,vObj3),'*m--');
%         1:maxIter,abs(vObj2-sigma),'.r-.');
set(gca,'Fontsize',fontsize,'YScale','log','Xlim',[1,xlim]);
if exportFlag
   set(h,'LineWidth',2);
   set(h(3),'Color',[1,0,0]);
   delete(h(2));
else
   set(h(2),'Color',[1,0.6,0.2]);
end;
legend(legendStr{:});
xlabel('k'); ylabel('Relative duality gap');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigRootConvNonneg4.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigRootConvNonneg4.eps');
end


figure(5);
h = plot(1:maxIter,cumsum(nIter1),'*b-', ...
         1:maxIter,cumsum(nIter2),'*r--', ...
         1:maxIter,cumsum(nIter3),'*m--');
set(gca,'Fontsize',fontsize,'Xlim',[1,xlim]);
if exportFlag
   set(h,'LineWidth',2);
   set(h(3),'Color',[1,0,0]);
   delete(h(2));
else
   set(h(2),'Color',[1,0.6,0.2]);
end;
legend(legendStr{:},'Location','NorthWest');
xlabel('k');

if exportFlag == 1
   ExportFigurePDF(prefixFig,'FigRootConvNonneg5.pdf');
elseif exportFlag == 2
   ExportFigureEPS(prefixFig,'FigRootConvNonneg5.eps');
else
   ylabel('Cumulative iteration count');
end


% ---------------------------------------------------------------------
function [result,data] = stoppingCriterion1(x,data)
% ---------------------------------------------------------------------
if isa(data.A,'function_handle')
   Ax  = data.A(x,1);
   r   = data.b - Ax;
   ATr = data.A(r,2);
else
   Ax  = data.A * x;
   r   = data.b - Ax;
   ATr = data.A' * r;
end

rNorm  = norm(r,2);
ytau   = r / rNorm;
lambda = data.dual_norm(ATr / rNorm);

delta = rNorm - data.b'*ytau + data.tau * lambda;

result = (delta / max(rNorm,1) < data.gamma);



% ---------------------------------------------------------------------
function [result,data] = stoppingCriterion2(x,data)
% ---------------------------------------------------------------------
if isa(data.A,'function_handle')
   Ax  = data.A(x,1);
   r   = data.b - Ax;
   ATr = data.A(r,2);
else
   Ax  = data.A * x;
   r   = data.b - Ax;
   ATr = data.A' * r;
end

rNorm  = norm(r,2);
ytau   = r / rNorm;
lambda = data.dual_norm(ATr / rNorm);

delta = rNorm - data.b'*ytau + data.tau * lambda;
eta   = delta / max(1,rNorm);

cond1 = delta < 1e-6;
cond2 = (rNorm < data.sigma) || (rNorm > data.sigma + 1.05*delta);

result = cond1 || cond2;



% ---------------------------------------------------------------------
function [result,data] = stoppingCriterion3(x,data)
% ---------------------------------------------------------------------
if isa(data.A,'function_handle')
   Ax  = data.A(x,1);
   r   = data.b - Ax;
   ATr = data.A(r,2);
else
   Ax  = data.A * x;
   r   = data.b - Ax;
   ATr = data.A' * r;
end

rNorm  = norm(r,2);
ytau   = r / rNorm;
lambda = data.dual_norm(ATr / rNorm);

delta = rNorm - data.b'*ytau + data.tau * lambda;
eta   = delta / max(1,rNorm);

cond1 = (eta < max(data.gamma * abs(rNorm - data.sigma), data.optmin));
cond2 = (rNorm < data.sigma) || (rNorm > data.sigma + 1.5*delta);

% 10*delta: the larger the constant, the harder it is to satisfy
%           The threshold seems to lie around 1.05*delta, below that
%           the method is likely to diverge

% fprintf(['                                                       ', ...
%          'gap = %9.3e, condition = [%d,%d]\n'], delta,cond1,cond2);

result = cond1 && cond2;



% ---------------------------------------------------------------------
function [varargout] = objective(x,data)
% ---------------------------------------------------------------------
if isa(data.A,'function_handle')
   Ax  = data.A(x,1);
   r   = data.b - Ax;
   ATr = data.A(r,2);
else
   Ax  = data.A * x;
   r   = data.b - Ax;
   ATr = data.A' * r;
end

f = r'*r / 2;
g = -ATr;

if nargout == 2
   varargout{1} = f;
   varargout{2} = data;
else
   varargout{1} = f;
   varargout{2} = g;
   varargout{3} = data;
end

options.project     = @NormL1NN_project;
options.primal_norm = @NormL1NN_primal;
options.dual_norm   = @NormL1NN_dual;



% ---------------------------------------------------------------------
function [vTau,vObj,vOpt,nIter] = findroot(A,b,sigma,options,maxIter,funGamma)
% ---------------------------------------------------------------------
nIter= NaN(maxIter,1);  % Number of iterations per subproblem
vTau = NaN(maxIter,1);  % Values of tau
vObj = NaN(maxIter,1);  % Objective values
vOpt = NaN(maxIter,1);  % Optimality tolerance values used

% Initialize data structure
data = struct();
data.A = A;
data.b = b;
data.tau = [];
data.dual_norm = @(x) NormL1NN_dual(x,1);
data.sigma = sigma;
data.optmin = 1e-11;

tau = 0; x = zeros(size(A,2),1); r = b;
for i=1:maxIter
   vTau(i) = tau;
   
   % Set objective and projection
   funObj  = @objective;
   funProj = @(x) NormL1NN_project(x,1,tau);
   
   % Set other fields
   data.tau   = tau;
   data.gamma = funGamma(i,r,sigma);

   % Invoke solver
   [x,info,data] = spgLasso(funObj, funProj, x, options, data);
   nIter(i) = info.iter;

   % Compute duality gap
   r = b - A * x;
   rNorm  = norm(r);
   lambda = data.dual_norm(A'*r)/rNorm;
   gap    = r'*(r-b) / rNorm + tau * lambda;
   
   % Record current objective and duality gap
   vObj(i) = rNorm;
   vOpt(i) = gap;

   % Stopping criterion
   if abs(rNorm - sigma) / max(1,sigma) < 1e-6
      break;
   end

   tau = tau + rNorm * (rNorm - sigma) / data.dual_norm(A'*r);
end
