function ExportFigureLaTeXEPS(prefix,filename,widthStr)

% Remove .eps extension
if strcmp(lower(filename(end-3:end)),'.eps')
   filename = filename(1:end-4);
end


% Analyze and extract all LaTeX strings
h = findall(gcf,'Type','text');
dictionary = cell(0,3);

for i=1:length(h)
   s = get(h(i),'String');
   if isLaTeXString(s)
      code = sprintf('xf%02d',i);
      set(h(i),'String',code);

      posh = get(h(i),'HorizontalAlignment');
      posv = get(h(i),'VerticalAlignment');
      switch lower(posh)
         case {'left'},  posh = 'l';
         case {'center'},posh = 'c';
         case {'right'}, posh = 'r';
      end
      switch lower(posv)
         case {'top','cap'}, posv = 't';
         case {'middle'},    posv = 'c';
         case {'baseline'},  posv = 'B';
         case {'bottom'},    posv = 'b';
      end

      dictionary{end+1,1} = code;
      dictionary{end,  2} = s;
      dictionary{end,  3} = [posv posh];
      dictionary{end,  4} = h(i);
   end
end

% Export LaTeX file
result = ExportLaTeX(prefix,filename,dictionary,widthStr);

% Restore labels
for i=1:size(dictionary,1)
   set(dictionary{i,4},'String',dictionary{i,2});
end

% --------------------------------------------------------------
function r = isLaTeXString(str)
% --------------------------------------------------------------
n = sum(double(str == '$'));

if n > 0
   r = mod(n+1,2);
else
   r = 0;
end


% --------------------------------------------------------------
function result = ExportLaTeX(prefix,basename,dictionary,width)
% --------------------------------------------------------------

statusEPS   = 0;
statusTEX   = 0;
statusDVI   = 0;
statusPS    = 0;
statusFilter= 0;
statusPDF   = 0;
statusCrop  = 0;
result      = 0;
errmsg      = '';

p = pwd;
try
   cd(prefix);

   % -------------------------------------------------------------------
   % Export plot to encapsulated Postscript
   % -------------------------------------------------------------------
   filenameEPS = sprintf('%s.tmp.eps',basename);
   if exist(filenameEPS,'file') ~= 0
      errmsg = sprintf('Warning: Intermediate file %s already exists', filenameEPS);
      error(errmsg);
   end
   
   try
      cmd = sprintf('print -depsc %s', filenameEPS);
      evalc(cmd);
      statusEPS = 1;
   catch
      errmsg = sprintf('Error: Unable to export intermediate file %s', filenameEPS);
      error(errmsg);
   end
   

   % -------------------------------------------------------------------
   % Create LaTex file
   % -------------------------------------------------------------------
   filenameTEX = sprintf('%s.tex',basename);
   if exist(filenameTEX,'file')
      errmsg = sprintf('Warning: Intermediate file %s already exists', filenameTEX);
      error(errmsg);
   end
   
   fp = fopen(filenameTEX,'w');
   if fp == -1
      errmsg = sprintf('Error: Unable to open intermediate file %s', filenameTEX);
      error(errmsg);
   else
      statusTEX = 1;
   end
   
   try
      fprintf(fp,'\\documentclass{article}\n');
      fprintf(fp,'\\usepackage{psfrag}\n');
      fprintf(fp,'\\usepackage{graphicx}\n');
      fprintf(fp,'\\begin{document}\n');
      fprintf(fp,'\\thispagestyle{empty}\n');

      for i=1:size(dictionary,1)
         fprintf(fp,'\\psfrag{%s}[%s]{%s}\n', ...
                 dictionary{i,1},dictionary{i,3},dictionary{i,2});
      end
      fprintf(fp,'\\includegraphics[width=%s]{./%s}\n',width,filenameEPS);
      fprintf(fp,'\\end{document}\n');

      fclose(fp);
   catch
      errmsg = sprintf('Error: Unable to write to intermediate file %s',filenameTEX);
      error(errmsg);
   end

   
   % -------------------------------------------------------------------
   % Compile with LaTeX to DVI
   % -------------------------------------------------------------------
   filenamesDVI = {sprintf('%s.dvi',basename), ...
                   sprintf('%s.aux',basename), ...
                   sprintf('%s.log',basename)};
   for i=1:length(filenamesDVI)
      if exist(filenamesDVI{i},'file') ~= 0
         errmsg = sprintf('Warning: Intermediate file %s already exists', filenamesDVI{i});
         error(errmsg);
      end
   end
   
   statusDVI = 1;
   [status,result] = system(sprintf('latex -halt-on-error %s', filenameTEX));
   if status ~= 0
      fprintf('%s\n',repmat('-',1,get(0,'CommandWindowSize')*[1;0]));
      fprintf('%s\n',result);
      fprintf('%s\n',repmat('-',1,get(0,'CommandWindowSize')*[1;0]));
      errmsg = sprintf('Error: LaTeX returned the above error; check tags');
      error(errmsg);
   end

   % -------------------------------------------------------------------
   % Convert DVI to PS
   % -------------------------------------------------------------------
   filenamePS = sprintf('%s.ps',basename);
   if exist(filenamePS,'file') ~= 0
      errmsg = sprintf('Warning: Intermediate file %s already exists', filenamePS);
      error(errmsg);
   end
   
   [status,result] = system(sprintf('dvips %s',filenamesDVI{1}));
   if status ~= 0
      fprintf('%s\n',repmat('-',1,get(0,'CommandWindowSize')*[1;0]));
      fprintf('%s\n',result);
      fprintf('%s\n',repmat('-',1,get(0,'CommandWindowSize')*[1;0]));
      errmsg = sprintf('Error: Conversion from DVI to PS returned the above error');
      error(errmsg);
   end;
   statusPS = 1;

   % -------------------------------------------------------------------
   % Filter PS file to get final EPS file
   % -------------------------------------------------------------------
   filenameFinal = sprintf('%s.eps',basename);
      
   fp = fopen(sprintf('%s',filenamePS),'r');
   if fp == -1
      errmsg = sprintf('Error: Unable to open intermediate file %s', filenamePS);
      error(errmsg);
   end
     
   fp2 = fopen(sprintf('%s',filenameFinal),'w');
   if fp2 == -1
      fclose(fp);
      errmsg = sprintf('Error: Unable to open intermediate file %s', filenameFinal);
      error(errmsg);
   else
      statusFilter = 1;
   end

   % Filter out unwanted lines
   suppress = 0; err = [];
   try
      while 1
         str = fgetl(fp);
         if isnumeric(str), break; end;

         if strncmpi(strtrim(str),'/Hide PSfrag',12)
            suppress = suppress + 1;
         end

         if (suppress <= 0)
            fprintf(fp2,'%s\n', str);
         end

         if strncmpi(strtrim(str),'/Unhide PSfrag',14)
            suppress = suppress - 1;
         end
      end
   catch
      err = lasterror;
   end

   % Close files
   fclose(fp);
   fclose(fp2);
   
   % Rethrow any error is needed
   if ~isempty(err), rethrow(err); end;

   % -------------------------------------------------------------------
   % SUCCESS
   % -------------------------------------------------------------------
   result = 1;

catch
   err = lasterror;
   if ~isempty(errmsg)
      fprintf('%s\n',errmsg);
   else
      fprintf('An unexpected error occurred:\n');
      fprintf('%s\n',err.message);
   end
end

% ------------------------
% Clean up temporary files
% ------------------------
if statusEPS
   deleteFile(filenameEPS);
end

if statusTEX
   deleteFile(filenameTEX);
end

if statusDVI
   for i=1:length(filenamesDVI)
      deleteFile(filenamesDVI{i});
   end
end
   
if statusPS
   deleteFile(filenamePS);
end

% -----------------------
% Return to original path
% -----------------------
cd(p);


% ----------------------------------------------------------------------
function deleteFile(filename)
% ----------------------------------------------------------------------

% Some files may not exists
if exist(filename,'file')
   try
      delete(filename);
   catch
      fprintf('Warning: Cleaning up intermediate file %s failed\n',filename);
   end
else
% Nothing
end      
